<?php
namespace Teknisa\Model\Teknisa\Repositories;

use Doctrine\DBAL\Connection;

class Licencauso extends \Doctrine\ORM\EntityRepository {

    public function findUseLicenseOpenBillingsProduct($clientId, $productLineId, $conditions, $order = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT LL.CDCLIENTE, LL.CDLINPCLIE, LL.NRLICENCAUSO, LU.CDLOCALUSO, LU.NMFANTLUSO,
                           LU.NMRAZSOCLUSO, LU.NMCONTLUSO, LU.NRTELELUSO, LU.DTLIBEPERSLUSO AS DTLIBEPERSLUSO,
                           NVL(LL.DTVALPERSLUPAGA, LL.DTVALPERSLUPEND) AS DTVALPERS,
                           DECODE(SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, SYSDATE, 'ATIVA'), 'ATIVA', 'Sim', 'Não') ATIVA,
                           DECODE(SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, SYSDATE, 'VALIDA'), 'VALIDA', 'Sim', 'Não') VALIDA
                      FROM LICENCAUSO LU, LINPLUSO LL
                     WHERE LL.CDCLIENTE = :CDCLIENTE
                       AND LL.CDLINPCLIE = :CDLINPCLIE
                       AND LU.CDCLIENTE = LL.CDCLIENTE
                       AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
                  ORDER BY LU.CDLOCALUSO)
             WHERE 1 = 1
             $conditions
             $order

SQL;
        $params = array(
            'CDCLIENTE' => $clientId,
            'CDLINPCLIE' => $productLineId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUseLicenseOpenBillingsAccessPoint($clientId, $productLineId, $conditions, $order = '')
    {
        $query = <<<SQL
        SELECT *
          FROM (SELECT LL.CDCLIENTE, LL.CDLINPCLIE, LL.NRLICENCAUSO, LU.CDLOCALUSO, LU.NMFANTLUSO,
                      LU.NMRAZSOCLUSO, LU.NMCONTLUSO, LU.NRTELELUSO, LU.DTLIBEPERSLUSO AS DTLIBEPERSLUSO,
                      (SELECT MIN(NVL(CX.DTVALPERSCXPAGO, CX.DTVALPERSCXPEND))
                         FROM CAIXALUSO CX
                        WHERE CX.CDCLIENTE = LL.CDCLIENTE
                          AND CX.NRLICENCAUSO = LL.NRLICENCAUSO) DTVALPERS,
                      (SELECT COUNT(*)
                         FROM CAIXALUSO CX
                        WHERE CX.CDCLIENTE = LL.CDCLIENTE
                          AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
                          AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, SYSDATE, 'ATIVA') = 'ATIVA') ATIVA,
                      (SELECT COUNT(*)
                         FROM CAIXALUSO CX
                        WHERE CX.CDCLIENTE = LL.CDCLIENTE
                          AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
                          AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, SYSDATE, 'VALIDA') = 'VALIDA') VALIDA
                 FROM LICENCAUSO LU, LINPLUSO LL
                WHERE LL.CDCLIENTE = :CDCLIENTE
                  AND LL.CDLINPCLIE = :CDLINPCLIE
                  AND LU.CDCLIENTE = LL.CDCLIENTE
                  AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
             ORDER BY LU.CDLOCALUSO)
         WHERE 1 = 1
         $conditions
         $order
SQL;
        $params = array('CDCLIENTE' => $clientId, 'CDLINPCLIE' => $productLineId);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function validateLinpluso($clientId, $nrlista)
    {
        $query = <<<SQL
      SELECT MAX(CL.CDCLIENTE)
        FROM LINPLUSO LL
        JOIN CONTRATOLINP CL ON (CL.DTCONTLINPRESC IS NULL AND LL.CDCLIENTE = :P_CDCLIENTE AND CL.CDLINPCLIE = LL.CDLINPCLIE)
        JOIN LICENCAUSO LUS ON ( LUS.CDCLIENTE = :P_CDCLIENTE AND  LL.NRLICENCAUSO  = LUS.NRLICENCAUSO )
        JOIN LISTA LIS ON (LIS.NRLISTA = :P_NRLISTA AND LUS.NRLICENCAUSO = LIS.CDLISTA )
       WHERE LL.CDCLIENTE     = :P_CDCLIENTE
         AND LL.IDLINPEXCLUIDA <> 'S'
    GROUP BY CL.CDCLIENTE
SQL;
        $params = array(
            'P_CDCLIENTE' => $clientId,
            'P_NRLISTA' => $nrlista
            );
        $paramTypes = array(
            'P_NRLICENCAUSO' => Connection::PARAM_STR_ARRAY
        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params,$paramTypes);
        return !empty($return);
    }

    public function findUseLicenseReferringToMonthProduct($clientId, $productLineId, $billingMonth, $conditions, $order = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT CDCLIENTE, CDLINPCLIE, NRLICENCAUSO, CDLOCALUSOAPUR AS CDLOCALUSO,
                           NMFANTLUSOAPUR AS NMFANTLUSO, NMRAZSOCLUSOAPU AS NMRAZSOCLUSO,
                           DECODE(IDLUSOATIVAPUR, 'S', 'Sim', 'Não') AS ATIVA, NMCONTLUSOAPUR AS NMCONTLUSO, NRTELELUSOAPUR AS NRTELELUSO,
                           DTLIBEPERSAPUR AS DTLIBEPERSLUSO, DTVALIPERSAPUR AS DTVALPERS,
                           CASE WHEN (IDLUSOATIVAPUR = 'S' OR DTVALIPERSAPUR > ADD_MONTHS(LAST_DAY(TRUNC(DTMESAPURLINP)), -1))
                                THEN 'Sim'
                                ELSE 'Não'
                            END AS VALIDA
                      FROM LINPLUSOAPUR
                     WHERE CDCLIENTE = :P_CDCLIENTE
                       AND CDLINPCLIE = :P_CDLINPCLIE
                       AND DTMESAPURLINP = :P_DTMESAPURLINP
                  ORDER BY CDLOCALUSOAPUR)
             WHERE 1 = 1
             $conditions
             $order
SQL;
        $params = array(
            'P_CDCLIENTE' => $clientId,
            'P_CDLINPCLIE' => $productLineId,
            'P_DTMESAPURLINP' => $billingMonth
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUseLicenseReferringToMonthAccessPoint($clientId, $productLineId, $billingMonth, $conditions, $order = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT LL.CDCLIENTE, LL.CDLINPCLIE, LL.NRLICENCAUSO,
                          LL.CDLOCALUSOAPUR AS CDLOCALUSO, LL.NMFANTLUSOAPUR AS NMFANTLUSO, LL.NMRAZSOCLUSOAPU AS NMRAZSOCLUSO,
                          LL.IDLUSOATIVAPUR, LL.NMCONTLUSOAPUR AS NMCONTLUSO, LL.NRTELELUSOAPUR AS NRTELELUSO,
                          LL.DTLIBEPERSAPUR AS DTLIBEPERSLUSO, LL.DTVALIPERSAPUR DTVAL ,
                          (SELECT MIN(CX.DTVALIPERCXAPUR)
                             FROM CAIXALUSOAPUR CX
                            WHERE CX.CDCLIENTE = LL.CDCLIENTE
                              AND CX.CDLINPCLIE = LL.CDLINPCLIE
                              AND CX.DTMESAPURLINP = LL.DTMESAPURLINP
                              AND CX.NRLICENCAUSO = LL.NRLICENCAUSO) AS DTVALPERS,
                          (SELECT COUNT(*)
                             FROM CAIXALUSOAPUR CX
                            WHERE CX.CDCLIENTE = LL.CDCLIENTE
                              AND CX.CDLINPCLIE = LL.CDLINPCLIE
                              AND CX.DTMESAPURLINP = LL.DTMESAPURLINP
                              AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
                              AND LL.IDLUSOATIVAPUR = 'S') ATIVA,
                          (SELECT COUNT(*)
                             FROM CAIXALUSOAPUR CX
                            WHERE CX.CDCLIENTE = LL.CDCLIENTE
                              AND CX.CDLINPCLIE = LL.CDLINPCLIE
                              AND CX.DTMESAPURLINP = LL.DTMESAPURLINP
                              AND CX.NRLICENCAUSO = LL.NRLICENCAUSO
                              AND (LL.IDLUSOATIVAPUR = 'S' OR CX.DTVALIPERCXAPUR > ADD_MONTHS(LAST_DAY(TRUNC(LL.DTMESAPURLINP)), -1))) VALIDA
                     FROM LINPLUSOAPUR LL
                    WHERE LL.CDCLIENTE = :P_CDCLIENTE
                      AND LL.CDLINPCLIE = :P_CDLINPCLIE
                      AND LL.DTMESAPURLINP = :P_DTMESAPURLINP
                 ORDER BY LL.CDLOCALUSOAPUR)
             WHERE 1 = 1
             $conditions
             $order
SQL;
        $params = array(
            'P_CDCLIENTE' => $clientId,
            'P_CDLINPCLIE' => $productLineId,
            'P_DTMESAPURLINP' => $billingMonth
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getCustomizationFiles($cdCliente, $filter, $page, $pageSize,$orderBy = ''){
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $query = <<<SQL
SELECT IDTIPLINPC, CDCLIENTE,CDLOCALUSO,NMFANTLUSO,NRINSESTLUSO,NRINSJURLUSO,IDBLOQLINP,IDLIBGERPER,NRINSMUNILUSO,IDTPIJURLUSO,
                          NMDEPTOLUSO, TO_CHAR (DTLIBEPERSLUSO, 'DD/MM/YYYY') AS DTLIBEPERSLUSO,TO_CHAR(DTVALIPERSLUSO, 'DD/MM/YYYY') AS DTVALIPERSLUSO,CDPAIS,SGESTADO,CDMUNICIPIO,NMMUNICIPIO,NMPAIS,IDMODPROC,IDLOCAPROC,
                          IDAMBILUSO,NMRAZSOCLUSO,IDLUSOATIV,DSENDELUSO,NMBAIRLUSO,NRCEPLUSO,NRTELELUSO,NMCONTLUSO,DTINCLPRIMPER, IDFATURAFILISEP,
                          DSEMAILENVLU,DSEMAILCOMLU,CDREPRLEGALEMPR,CDLINPCLIE,NMLINPCLIE

                    FROM
                      (SELECT LIN.IDTIPLINPC, LUS.CDCLIENTE,LUS.IDTPIJURLUSO,LUS.NRLICENCAUSO,LUS.CDLOCALUSO,LUS.NMFANTLUSO,LUS.NRINSESTLUSO,LUS.NRINSJURLUSO,
                        LIN.CDLINPCLIE,LIN.NMLINPCLIE,LIN.IDBLOQLINP,CLI.IDLIBGERPER,LUS.NRINSMUNILUSO,LUS.NMDEPTOLUSO,
                        LUS.DTLIBEPERSLUSO,LUS.DTVALIPERSLUSO,
                        LUS.CDPAIS,LUS.SGESTADO,LUS.CDMUNICIPIO,M.NMMUNICIPIO,P.NMPAIS,LUS.IDMODPROC,LUS.IDLOCAPROC,LUS.IDAMBILUSO,LUS.NMRAZSOCLUSO,LUS.IDLUSOATIV,
                        LUS.DSENDELUSO,LUS.NMBAIRLUSO,LUS.NRCEPLUSO,LUS.NRTELELUSO,LUS.NMCONTLUSO,LUS.DTINCLPRIMPER,
                        NVL(LUS.DSEMAILENVLU, CLI.DSEMAILENVCL) AS DSEMAILENVLU,NVL(LUS.DSEMAILCOMLU, CLI.DSEMAILCOMCL) AS DSEMAILCOMLU,
                        NVL(LUS.CDREPRLEGALEMPR, 'N') AS CDREPRLEGALEMPR, LUS.IDFATURAFILISEP
                      FROM
                        (SELECT DISTINCT( LUS.NRLICENCAUSO ),
                          LUS.CDCLIENTE
                        FROM LICENCAUSO LUS
                        WHERE ( LUS.CDCLIENTE = :P_CDCLIENTE )
                        UNION
                        SELECT DISTINCT( LUS.NRLICENCAUSO ),
                          LUS.CDCLIENTE
                        FROM LICENCAUSO LUS,
                          CLIENTE C,
                          LINPLUSO LI
                        WHERE ( LUS.CDCLIENTE                            = :P_CDCLIENTE )
                        AND ( LUS.CDCLIENTE                              = C.CDCLIENTE )
                        AND ( LUS.CDCLIENTE                              = LI.CDCLIENTE )
                        AND ( LUS.NRLICENCAUSO                           = LI.NRLICENCAUSO )
                        AND ( LUS.IDLUSOATIV                            IN ( 'S', 'P' ) )
                        AND ( NOT LUS.NMFANTLUSO                        IS NULL )
                        AND ( NOT LUS.IDMODPROC                         IS NULL )
                        AND ( NOT LUS.NMRAZSOCLUSO                      IS NULL )
                        AND ( NOT LUS.NMCONTLUSO                        IS NULL )
                        AND ( NOT LUS.DSENDELUSO                        IS NULL )
                        AND ( NOT LUS.NMBAIRLUSO                        IS NULL )
                        AND ( NOT LUS.NRCEPLUSO                         IS NULL )
                        AND ( NOT LUS.CDPAIS                            IS NULL )
                        AND ( NOT LUS.SGESTADO                          IS NULL )
                        AND ( NOT LUS.CDMUNICIPIO                       IS NULL )
                        AND ( NOT NVL(LUS.DSEMAILCOMLU, C.DSEMAILCOMCL) IS NULL )
                        ) FIL,
                        LICENCAUSO LUS,
                        CLIENTE CLI,
                        (SELECT CL.CDCLIENTE,
                          LL.NRLICENCAUSO,
                          LISTAGG(CL.CDLINPCLIE, ',') WITHIN GROUP (ORDER BY CL.CDLINPCLIE) AS CDLINPCLIE,
                          LISTAGG(CL.NMLINPCLIE, ', ') WITHIN GROUP (ORDER BY CL.NMLINPCLIE) AS NMLINPCLIE,
                          LISTAGG(CL.IDTIPLINPC, ',') WITHIN GROUP (ORDER BY CL.IDTIPLINPC) AS IDTIPLINPC,
                          CL.IDBLOQLINP
                        FROM LINPLUSO LL,
                          CONTRATOLINP CL
                        WHERE LL.CDCLIENTE     = :P_CDCLIENTE
                        AND CL.CDCLIENTE       = LL.CDCLIENTE
                        AND CL.CDLINPCLIE      = LL.CDLINPCLIE
                        AND CL.DTCONTLINPRESC IS NULL
                        AND LL.IDLINPEXCLUIDA <> 'S'
                        GROUP BY CL.IDBLOQLINP,CL.CDCLIENTE,LL.NRLICENCAUSO
                        ) LIN,
                        PAIS P,
                        MUNICIPIO M
                      WHERE ( LUS.CDCLIENTE      = :P_CDCLIENTE )
                      AND P.CDPAIS               = LUS.CDPAIS
                      AND M.CDMUNICIPIO          = LUS.CDMUNICIPIO
                      AND M.SGESTADO             = LUS.SGESTADO
                      AND M.CDPAIS               = LUS.CDPAIS
                      AND ( CLI.CDCLIENTE        = LUS.CDCLIENTE )
                      AND ( LIN.CDCLIENTE (+)    = LUS.CDCLIENTE )
                      AND ( LIN.NRLICENCAUSO (+) = LUS.NRLICENCAUSO )
                      AND ( FIL.CDCLIENTE        = LUS.CDCLIENTE )
                      AND ( FIL.NRLICENCAUSO     = LUS.NRLICENCAUSO )
                      )
                    WHERE 1 = 1
                    $filter
                    ORDER BY $orderBy
SQL;



        $params = array(
            'P_CDCLIENTE' => $cdCliente,
        );
        $query = $this->getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query ,$params);
    }

    private function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $max = $offset + $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }

        return $query;
    }
}
